#!/usr/bin/python3

class Preferences:
	def __init__(self) -> None:
		self.link = None
		self.song_metadata: dict = None
		self.quality_download = None
		self.output_dir = None
		self.ids = None
		self.json_data = None
		self.recursive_quality = None
		self.recursive_download = None
		self.not_interface = None
		self.method_save = None
		self.make_zip = None